# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-02-13 09:20
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('sites', '0002_alter_domain_unique'),
        ('blogging', '0004_auto_20150504_1826'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='category',
            options={'ordering': ['name'], 'verbose_name': 'category', 'verbose_name_plural': 'categories'},
        ),
        migrations.AlterModelOptions(
            name='post',
            options={'ordering': ['-published_on'], 'verbose_name': 'item', 'verbose_name_plural': 'items'},
        ),
        migrations.AddField(
            model_name='post',
            name='post_type',
            field=models.IntegerField(choices=[(1, 'Text'), (2, 'Quote'), (3, 'Link'), (4, 'Video')], db_index=True, default=1, verbose_name='Type'),
        ),
        migrations.AddField(
            model_name='post',
            name='source',
            field=models.URLField(blank=True, null=True, verbose_name='Post source'),
        ),
        migrations.AlterField(
            model_name='category',
            name='name',
            field=models.CharField(max_length=100, verbose_name='Name'),
        ),
        migrations.AlterField(
            model_name='picture',
            name='name',
            field=models.CharField(max_length=100, verbose_name='Name'),
        ),
        migrations.AlterField(
            model_name='post',
            name='author',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='Author'),
        ),
        migrations.AlterField(
            model_name='post',
            name='categories',
            field=models.ManyToManyField(to='blogging.Category', verbose_name='Categories'),
        ),
        migrations.AlterField(
            model_name='post',
            name='comments_open',
            field=models.BooleanField(default=True, verbose_name='Are comments open?'),
        ),
        migrations.AlterField(
            model_name='post',
            name='content',
            field=models.TextField(verbose_name='Content'),
        ),
        migrations.AlterField(
            model_name='post',
            name='published_on',
            field=models.DateTimeField(db_index=True, verbose_name='Published on'),
        ),
        migrations.AlterField(
            model_name='post',
            name='selected',
            field=models.BooleanField(default=False, verbose_name='Selected'),
        ),
        migrations.AlterField(
            model_name='post',
            name='slug',
            field=models.SlugField(max_length=150, verbose_name='Slug'),
        ),
        migrations.AlterField(
            model_name='post',
            name='status',
            field=models.IntegerField(choices=[(1, 'Draft'), (2, 'Published'), (3, 'Deleted')], db_index=True, default=1, verbose_name='Status'),
        ),
        migrations.AlterField(
            model_name='post',
            name='title',
            field=models.CharField(max_length=150, verbose_name='Title'),
        ),
        migrations.AlterUniqueTogether(
            name='post',
            unique_together=set([('site', 'slug')]),
        ),
    ]



# -*- coding: utf-8 -*-
from collections import OrderedDict

from django.contrib import admin

from blogging.actions import (make_draft, make_post_type_action,
                              make_published, make_selected, update_counters)
from blogging.models import Category, Picture, Post


@admin.register(Picture)
class PictureAdmin(admin.ModelAdmin):
    prepopulated_fields = {"slug": ("name",)}
    list_display = (
        'name', 'site', 'description', 'image',
    )
    list_filter = ('site',)
    search_fields = ('name', 'description')


@admin.register(Category)
class CategoryAdmin(admin.ModelAdmin):
    prepopulated_fields = {"slug": ("name",)}
    list_display = (
        'name', 'site', 'visible_posts_count', 'all_posts_count',
    )
    list_filter = ('site',)
    search_fields = ('name',)
    actions = [update_counters, ]


@admin.register(Post)
class PostAdmin(admin.ModelAdmin):
    list_display = (
        'title', 'author', 'status', 'published_on', 'selected', 'post_type',
        'site'
    )
    list_filter = [
        'site', 'author', 'status', 'selected', 'categories', 'post_type'
    ]
    date_hierarchy = 'published_on'
    prepopulated_fields = {"slug": ("title",)}
    search_fields = ('excerpt', 'content', 'item__title')
    filter_horizontal = ["categories"]

    def get_actions(self, request):
        actions_list = [
            ('make_published', (make_published, 'make_published', make_published.short_description)),
            ('make_draft', (make_draft, 'make_draft', make_draft.short_description)),
            ('make_selected', (make_selected, 'make_selected', make_selected.short_description)),
        ]
        for k, v in Post.CONTENT_TYPE_CHOICES:
            actions_list.append(make_post_type_action(k, v))
        return OrderedDict(actions_list)

    def formfield_for_manytomany(self, db_field, request, **kwargs):
        """
        Monkey patching the form field for categories.

        TODO: Create a widget to manage it more easily
        """
        field = super(PostAdmin, self).formfield_for_manytomany(db_field, request, **kwargs)
        field.queryset = field.queryset.order_by('site__domain')
        field.label_from_instance = lambda obj: "%(site)s - %(name)s" % {
            'site': obj.site, 'name': obj.name
        }
        return field



import os
local_path = lambda path: os.path.join(os.path.dirname(__file__), path)

DEBUG = True
ALLOWED_HOSTS = ['*']

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': 'db.sqlite3',
        'TEST_NAME': ':memory:'
    }
}

SITE_ID = 1

INSTALLED_APPS = [
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.sites',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'blogging',
    'tests',
]

MEDIA_URL = '/media/'

MEDIA_ROOT = local_path('media')

STATIC_URL = '/static/'

SECRET_KEY = "app-blogging"


TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [
            local_path('templates'),
        ],
        'APP_DIRS': True,
        'OPTIONS': {
            # ... some options here ...
            'debug': True,  # the former TEMPLATE_DEBUG
            'context_processors': [
                # Insert your TEMPLATE_CONTEXT_PROCESSORS here or use this
                # list if you haven't customized them:
                'django.contrib.auth.context_processors.auth',
                'django.template.context_processors.debug',
                'django.template.context_processors.i18n',
                'django.template.context_processors.media',
                'django.template.context_processors.static',
                'django.template.context_processors.request',
                'django.template.context_processors.tz',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

MIDDLEWARE_CLASSES = (
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.locale.LocaleMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.auth.middleware.SessionAuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
    'django.middleware.locale.LocaleMiddleware',
)

ROOT_URLCONF = 'tests.urls'



# -*- coding: utf-8 -*-
from django.conf import settings
from django.conf.urls import include, url
from django.contrib import admin

urlpatterns = [

    url(r'^admin/', include(admin.site.urls)),
    url(r'^', include('blogging.urls')),
]



# -*- coding: utf-8 -*-
"""Models for blogging app."""
from __future__ import unicode_literals

import hashlib
import time

from django.conf import settings
from django.contrib.auth.models import User
from django.contrib.sites.managers import CurrentSiteManager
from django.contrib.sites.models import Site
from django.core.urlresolvers import NoReverseMatch, reverse
from django.db import models
from django.utils import timezone
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext as _

from blogging.managers import (AvailableCategoriesManager,
                               AvailableItemsManager, PostManager)


def upload_to_blogging(instance, filename):
    """Create file path."""
    hasher = hashlib.md5()
    hasher.update(filename.encode('utf-8'))
    hasher.update(str(time.time()).encode('utf-8'))
    hashed_name = hasher.hexdigest()
    extension = filename.split('.')[-1]
    return "blogging/pictures/{0}-{1}.{2}".format(
        hashed_name[:5], instance.slug, extension
    )


@python_2_unicode_compatible
class Picture(models.Model):
    name = models.CharField(_("Name"), max_length=100)
    slug = models.SlugField(_("Slug"))
    description = models.TextField(_("Description"), blank=True)
    site = models.ForeignKey(Site, verbose_name=_("Site"), default=settings.SITE_ID)
    image = models.ImageField(upload_to=upload_to_blogging, max_length=200)

    def __str__(self):
        """Human picture name."""
        return self.name


@python_2_unicode_compatible
class Category(models.Model):
    """A category to regroup similar articles."""

    name = models.CharField(_("Name"), max_length=100)
    slug = models.SlugField(_("Slug"))
    description = models.TextField(_("Description"), blank=True)
    site = models.ForeignKey(Site, verbose_name=_("Site"), default=settings.SITE_ID)
    picture = models.ForeignKey(Picture, verbose_name=_("Picture"), blank=True, null=True)

    # hidden cached field
    visible_posts_count = models.IntegerField(_("Visible posts in category"), editable=False, default=0)
    all_posts_count = models.IntegerField(_("All posts in category"), editable=False, default=0)

    objects = models.Manager()  # The default manager.
    on_site = CurrentSiteManager()
    availables = AvailableCategoriesManager()  # The Online manager.

    class Meta:
        ordering = ['name']
        verbose_name = _("category")
        verbose_name_plural = _("categories")

    def __str__(self):
        """Human category name."""
        return self.name

    def get_absolute_url(self):
        return reverse('blog-category', args=[self.slug])

    def natural_key(self):
        return [self.slug, self.site.id]

    def is_empty(self):
        return not Post.availables.published().filter(categories=self).exists()

    @property
    def get_online_posts_count(self):
        return Post.availables.published().filter(categories=self).count()

    @property
    def get_all_posts_count(self):
        return Post.availables.published().filter(categories=self).count()

    def update_counters(self):
        self.visible_posts_count = self.get_online_posts_count
        self.all_posts_count = self.get_all_posts_count
        self.save()


@python_2_unicode_compatible
class Post(models.Model):
    """
    The Post contains the generic fields for a blog item.

    It's for example the publication date, the author, the slug, etc.
    """

    # Constants for the blog status
    DRAFT = 1
    PUBLISHED = 2
    DELETED = 3
    STATUS_CHOICES = (
        (DRAFT, _("Draft")),
        (PUBLISHED, _("Published")),
        (DELETED, _("Deleted")),
    )
    # Constants for content type
    TEXT = 1
    QUOTE = 2
    LINK = 3
    VIDEO = 4
    CONTENT_TYPE_CHOICES = (
        (TEXT, _("Text")),
        (QUOTE, _("Quote")),
        (LINK, _("Link")),
        (VIDEO, _("Video")),
    )

    title = models.CharField(_(u"Title"), max_length=150)
    slug = models.SlugField(_(u"Slug"), max_length=150, db_index=True)
    author = models.ForeignKey(User, verbose_name=_(u"Author"))
    excerpt = models.TextField(_(u"Excerpt"), blank=True, db_column="exceprt")
    content = models.TextField(_(u"Content"))

    main_picture = models.ForeignKey(Picture, verbose_name=_("Picture"), blank=True, null=True)

    published_on = models.DateTimeField(_("Published on"), db_index=True,
                                        default=timezone.now)
    created_on = models.DateTimeField(auto_now_add=True, editable=False)
    updated_on = models.DateTimeField(auto_now=True, editable=False)
    post_type = models.IntegerField(_("Type"), choices=CONTENT_TYPE_CHOICES,
                                    default=TEXT, db_index=True)
    status = models.IntegerField(_("Status"), choices=STATUS_CHOICES,
                                 db_index=True, default=DRAFT)

    selected = models.BooleanField(_("Selected"), default=False)
    comments_open = models.BooleanField(_("Are comments open?"), default=True)
    source = models.URLField(_("Post source"), blank=True, null=True)
    categories = models.ManyToManyField(Category, verbose_name=_("Categories"))

    site = models.ForeignKey(Site, verbose_name=_("Site"), default=settings.SITE_ID)

    # Managers
    objects = PostManager()
    on_site = CurrentSiteManager()
    availables = AvailableItemsManager()  # The Online manager.

    class Meta:
        ordering = ['-published_on']
        verbose_name = _("item")
        verbose_name_plural = _("items")
        unique_together = (
            ("site", "slug"),
        )

    def __str__(self):
        """Human post name."""
        return self.title

    def natural_key(self):
        return [self.slug, self.site.id]

    def get_absolute_url(self):
        try:
            kwargs = {
                'year': self.published_on.strftime("%Y"),
                'month': self.published_on.strftime("%m"),
                'day': self.published_on.strftime("%d"),
                'slug': self.slug
            }
            return reverse('blog-item', kwargs=kwargs, urlconf=settings.ROOT_URLCONF)
        except NoReverseMatch:
            try:
                kwargs = {
                    'year': self.published_on.strftime("%Y"),
                    'month': self.published_on.strftime("%m"),
                    'slug': self.slug
                }
                return reverse('blog-item', kwargs=kwargs, urlconf=settings.ROOT_URLCONF)
            except NoReverseMatch:
                kwargs = {
                    'slug': self.slug
                }
                return reverse('blog-item', kwargs=kwargs, urlconf=settings.ROOT_URLCONF)

    def __item_cache_key(self):
        """Return a unique item key that can be used in order to cache it."""
        return "blogging:post:{0}".format(self.id)

    def related_items(self):
        """Return items related to the current item."""
        return Post.availables.related_items(self)



# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-02-14 00:37
from __future__ import unicode_literals

from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('blogging', '0005_auto_20170213_0920'),
    ]

    operations = [
        migrations.AlterField(
            model_name='post',
            name='published_on',
            field=models.DateTimeField(db_index=True, default=django.utils.timezone.now, verbose_name='Published on'),
        ),
    ]
